function [maxerr, rms_err, N] = IAMLS_sD(N, s, gridtype, D, maxlevel)
% function [maxerr, rms_err] = IAMLS_sD(N, s, gridtype, D, maxlevel)
% Computes sD iterated AMLS approximation
% Inputs:
% N: number of interpolation points
% s: space dimension
% gridtype: Type of data points: 'c'=Chebyshev, 'f'=fence(rank-1 lattice),
% 'h'=Halton, 'l'=latin hypercube, 'n'=random normal, 'r'=random uniform,
% 's'=Sobol, 'u'=uniform   
% D: Mazya/Schmidt shape parameter
% maxlevel: number of IAMLS iterations
% Outputs:
% maxerr: maximum error on set of M (set below) random points 
% rms_err: RMS error on set of M (set below) random points 
% N: actual number of data points used (could be smaller for 'c' and 'u'
% types)
% Calls on: rbf_definition, testfunctionsD, DistanceMatrixB

%%%%%%%%%%%%%%%%%% Editable Section %%%%%%%%%%%%%%%%%%%%

% Number of evaluation points for error computation
M = 500;      % to create evaluation grid in unit cube

% The RBF is defined in the function rbf_definition 
% (separate file rbf_definition has to be edited to get desired RBF, 
% however, this file can be accessed by all programs)
global rbf
rbf_definition;

%%%%%%%%%%%%%%%%%% Code below this line need not be changed %%%%%%%%%%%%%%%%

[datasites, N] = CreatePoints(N,s,gridtype);
% Let centers coincide with data sites
centers = datasites;
respoints = datasites;
rhs = testfunctionsD(datasites);

h = 1/(nthroot(N,s)-1);
ep = 1/(sqrt(D)*h)

% Create evaluation locations in the unit cube
[evalpoints, M] = CreatePoints(M,s,'r');
exact = testfunctionsD(evalpoints);

% Evaluate the test function at the data points
rhs = testfunctionsD(datasites);

t=cputime;
% Now we find the RBF interpolant (this should not require any changes)
% Compute the distance matrix between the data sites and centers
DM_data = DistanceMatrixB(datasites,centers);
IM = rbf(ep,DM_data)/(sqrt(pi*D)^s);
condA = cond(IM);

% Compute the distance matrix between the evaluation points and centers
DM_eval = DistanceMatrixB(evalpoints,centers);
EM = rbf(ep,DM_eval)/(sqrt(pi*D)^s);

t=cputime;
Pf = EM*rhs;
maxerr(1) = max(abs(Pf - exact));
rms_err(1) = norm(Pf-exact)/sqrt(M);
for level=2:maxlevel
    % Compute new residual
    rhs = rhs - IM*rhs;
    % Evaluate on evaluation points (for error computation and plotting)
    Pf = Pf + EM*rhs;
    % Compute maximum error on evaluation grid
    maxerr(level) = max(abs(Pf - exact));
    rms_err(level) = norm(Pf-exact)/sqrt(M);

end

leveltime = cputime-t;
disp(sprintf('Time: %f seconds', leveltime))

figure
semilogy(1:maxlevel,maxerr,'b',1:maxlevel,rms_err,'r');
legend('maxerr','rmserr')
